/*******************************************************************
 *            DISPLAY RAW CODES OF A COMMERCIAL IR HANDSET 
 *            ============================================
 *              
 * In this program the KY-022 IR LED receiver module is connected 
 * to port pin 2 of the Arduino Uno. The program receives data from
 * a commercial IR handset and displays the raw codes on serial monitor
 * 
 * Author: Dogan Ibrahim
 * Date  : April 2019
 * File  : IRhandset2
 *******************************************************************/
#include <IRremote.h>                         // Include IR header file

int RX_PIN = 2;                               // KY-022 S pin at pin 2 
IRrecv irrecv(RX_PIN);                        // Initialize library
decode_results results;                        

void setup() 
{ 
  Serial.begin(9600);                         // Start serial monitor 
  irrecv.enableIRIn();                        // Enable the IR receiver 
} 

//
// Display teh Raw codes
//
void display(decode_results *results)
{
  int count = results -> rawlen;              // Raw data length
//
// If known, display the protocol type
//  
  if (results->decode_type == NEC)
     Serial.print("NEC: ");
  else if(results->decode_type == SONY)
     Serial.print("SONY: ");
  else if(results->decode_type == RC5)
     Serial.print("RC5: ");
  else if(results->decode_type == RC6)
     Serial.print("RC6: ");
  else if(results->decode_type == JVC)
     Serial.print("JVC: ");
  else if(results->decode_type == PANASONIC)
     Serial.print("PANASONIC: ");
  else
     Serial.print("UNKNOWN: ");
//
// Display IR code and raw data
//     
   Serial.println(results->value, HEX);
   Serial.print("Raw data count = ");
   Serial.println(count, DEC);

   for (int i = 1; i < count; i++) 
   {
    if (i & 1) 
    {
      Serial.print("+ ");
      Serial.println(results->rawbuf[i]*USECPERTICK, DEC);
    }
    else 
    {
      Serial.print("- ");
      Serial.println((unsigned long) results->rawbuf[i]*USECPERTICK, DEC);
    }
  }
}
  

//
// Inside the main program loop, display the received IR codes in
// Raw format
//
void loop() 
{ 
 if (irrecv.decode(&results))                 // If code received
 {
    display(&results);                        // Display Raw codes
    irrecv.resume();                          // Wait for next value
 }
}
