
#define ANZ_MESSWERTE 5         // wie viele Messwerte werden gebraucht zur Mittelwertberechnung
#define R_VORWIDERSTAND 10000.0 // wie groß ist der Vorwiderstand (hier 10k)
#define MAX_ADC 1023.0          // bei 10Bit-Auflösung ist der ADC-wert 1023.0 
#define BETA 3380.0             // Wert aus dem Datenblatt Pollin Artikelnummer 220757
#define T_RAUM 298.15           // Raumtemperatur von 25°C in Kelvin
#define R_RAUM_TEMP  10000.0    // Wert des NTC bei Raumtemperatur von 25°C
#define WAIT_NEXT_LOOP 4000     // Wartezeit für nächsten Schleifendurchlauf
#define NTC_PIN A0              // an wlechem Analogport ist der NTC angeschlossen
#define WAIT_ADC 22             // Wartezeit zwischen dem Erfassen der einzelnen Spannungswerte

double v_temp = 0;              // Variable für aktuell berechnete Temperatur
double v_ntc = 0;               // aktueller Wert des NTC
double v_kelvin     = 0;        // aktueller Temperaturwert [K]
double v_celsius    = 0;         // aktuelle Temperatur in Celsius
double v_adc  = 0;              // Mittelwert aller gemessenen Spannungen
int    adc[ANZ_MESSWERTE];      // array der Spannungsmesswerte
unsigned char x;                      // Laufvariable für for-Schleife

// Funktionsdeklaration:
double lese_NTC(void); 

void setup() 
{ 
  Serial.begin(9600);// initialsiere die serielle Schnittstelle
}

void loop() 
{
  
  v_temp = lese_NTC();    // messe die Spannung und berechne die Temperatur
  
  if (v_temp > 20.0 && v_temp < 22.0) // ist die Temperatur OK?
  {
    Serial.print("Die Temperatur beträgt momentan angenehme : ");
    Serial.print(v_temp);Serial.println(" °C");
  } 
  else if (v_temp >= 24.0)          // ist es zu warm?
  {
    Serial.print("Die Raumtemperatur ist: ");
    Serial.print(v_temp);Serial.println(" °C, und damit zu warm!");
  } 
  else                              // oder ist es zu kalt?
  {
    Serial.print("Die Raumtemperatur ist momentan: ");
    Serial.print(v_temp);Serial.println("°C  Fröstel ... Fröstel");
  }
  delay(WAIT_NEXT_LOOP);                              // warte mal
}
// ######## E N D    O F    L O O P ##################################
// ###################################################################



double lese_NTC(void) 
{
  for (x=0; x < ANZ_MESSWERTE; x++) // messe die Spannung am NTC und speichere diese im array
  {
    adc[x] = analogRead(NTC_PIN); // messe die Spannung
    delay(WAIT_ADC);              // warte mit der nächsten Messung
  }
  
  for (x=0; x < ANZ_MESSWERTE; x++) // Summiere alle Messwerte
     { v_adc += adc[x];  }   
  v_adc /= ANZ_MESSWERTE;           // und bilde den Mittelwert
          // berechne aus der Spannung den Widerstandswert des NTC; Vntc = V[Rvor] * Rvor / (Rntc + Rvor) 
  v_ntc = R_VORWIDERSTAND * ( (MAX_ADC / v_adc) - 1);  // die obere Formel aufgelöst nach Rntc
                                                       
  // die Näherungsfromel zur Berechnung des Temperaturwertes anhand des berechneten Widerstandes für Rntc:  
  v_kelvin = (BETA * T_RAUM) / (BETA + (T_RAUM * log(v_ntc / R_RAUM_TEMP)));
  
  v_celsius = v_kelvin - 273.15;  // Umrechnen des Wertes in °C

  return v_celsius;    // gib den Wert an die aufrufende Funktion zurück
}
