#include <Arduino.h>
// Module connection pins (Digital Pins)
#define CLK 2
#define DIO 3
#define SW_s 5
#define SW_m 4
#define LSP 9  // PWM Ausgang für Ton, falls gewünscht
#define LED 13

#define LSP_Frequenz 4000 // mit welcher Frequnz soll der Piepston ertönen
#define LSP_Dauer 1200 // wie lange soll der Ton spielen

// The amount of time (in milliseconds) between tests
#define TEST_DELAY   2000
#define c_loop 20  // wie oft wird loop durchlaufen bis 1s vergangen ist
#define dauer 1000 // eine Sekunde
#define debounce 10 // wartezeit für nächsten Tastendruck
#define max_Minuten 90 // wie viele Minuten sollen Maximal eingestellt werden können
#define pos 0 // wird ausgegeben ab Position 0
#define len 4 // die Zahl ist 4 Byte lang
#define lead_zero 1 // führende 0 einfügen: ja
#define dots 0b01000000 // doppelpunkt anzeigen
#define no_dots 0b00000000 // keinen Doppelpunkt anzeigen
#define ON true
#define OFF false
#define t_key_in 500 // wie lange warten zwischen zwei Tastendrücken
#define t_display 2 // dieser Wert ist abhängig von der Anzahl der verwendeten Zeichen
                        // je mehr Zeichen desto kleiner soll die Zeit sein um flackern zu verhindern
                        // aber je größer die Zeit, desto heller die Anzeige
unsigned char digit[8]={0x80,0x40,0x20,0x10,0x08,0x04,0x02,0x01};
unsigned char data[14]={0x03,0x9F, 0x25, 0x0D, 0x99, 0x49, 0x41, 0x1F, 0x01, 0x09, 0xFE, 0xFD, 0xFF}; //0 1 2 3 4 5 6 7 8 9 0 . -  AUS 
unsigned char dig_active[8]={0,0,0,0,0,0,0,0};// es können nur maximal acht digit 1.. 8 aktiv sein, wenn 0xA dann aus ansonsten 0 ... 9 und B für Dot
unsigned char x;

unsigned long timer_akt,timer_key_akt;
unsigned long timer_old,timer_key_old;

uint8_t m, m1, m2, s, s1, s2, Zeit;
boolean taste_m, taste_s;
boolean dobledot;// soll der Punkt die Sekunde anzeigt
boolean T_runs; // Zeitzähler aktivieren
boolean key_in; // Merker, ob Tasten betätigt wurden 
boolean debug; // um mit einer Variablen steuern zu können, ob die serielle Schnittstelle aktiviert werden soll oder nicht
boolean done;
void taste_pressed(void); // schau ob eine Taste gedrückt ist
void fertig(void);        // wenn Zeit abgelaufen dann soll es piepsen
void dekrementiere_Zeit(void); // zähle 1 s herunter
void LED_blink(void);
void pollin_display(void);
 
//****************************************************************************************
//**************  S  E  T  U  P  ( )   ***************************************************
//****************************************************************************************  
void setup()
{
  //debug=true;     // wenn debug == true, dann Statusmeldungen über RS232
  debug=false;  // wenn debug ==false, keine Ausgabe von Statusmeldungen über RS232
  
  if(debug)
  { Serial.begin(9600); }
  
  pinMode(LED,OUTPUT);
  pinMode(DIO,OUTPUT);
  pinMode(CLK, OUTPUT);

  pinMode(SW_s, INPUT_PULLUP);
  pinMode(SW_m, INPUT_PULLUP);
  m = 0;
  s = 0;
  
  timer_old=millis();
  dobledot=0; // doppelpunkt ist ausgeschaltet
  T_runs=0;// timer ist nicht aktiv
  key_in=0;
  
}
//****************************************************************************************
//**************  L   O   O   P    ( ) ***************************************************
//****************************************************************************************  
void loop()
{
   pollin_display();
   if((!digitalRead(SW_m)) || (!digitalRead(SW_s)))  // wurde taste m oder s gedrückt?
      { taste_pressed();}

   timer_key_akt=millis();
   if((timer_key_akt > timer_key_old + t_key_in) && (key_in))     // wenn t_key_in [ms] keine Eingabe erfolgte, dann beginne Count Down
      { T_runs=1; }

   if(T_runs)// wenn der Timer aktiv ist die RestZeit aktualisieren und anzeigen
   {
      timer_akt=millis();
      if(timer_akt > timer_old + 500)
      { 
        dobledot = !dobledot;
        timer_old=millis();
        Serial.print('d');Serial.println(dobledot,DEC);
        if((dobledot) && (T_runs))
        {// nur wenn timer aktiv, soll Zeit angezeigt werden 
         // nun soll kein Punkt dargestellt werden
            Serial.print("bin hier :");Serial.print('s');    Serial.print(s,DEC);
            dekrementiere_Zeit();
            Serial.print('x');   Serial.println(s,DEC);
            Serial.print(" verbleibende Zeit: ");Serial.print(m,DEC);Serial.print(" Minuten und ");Serial.print(s,DEC);Serial.println(" Sekunden");
        }
      }  
  }
}
//#########################################################################
//####### e n d   o f      l o o p  #######################################
//#########################################################################

void pollin_display(void)
{
   //Ganze zahl
  m1=m / 10; // Zenerstelle_Minute
  // Rest
  m2=m % 10; // Einerstelle_Minute
  // ganze Zahl
  s1=s /10;  // Zenerstelle_Sekunde
  // Rest
  s2=s % 10; // Einerstelle_Sekunde

  if(T_runs)
  {
    dig_active[0]= data[m1]; // 
    dig_active[1]= data[m2]; // 
    dig_active[2]= 0xFF; // 
    dig_active[3]= 0xFF; // 
    dig_active[4]= 0xFF; // 
    dig_active[5]= 0xFF; // 
    dig_active[6]= data[s1]; // 
    dig_active[7]= data[s2]; // 
  }
  else
  {
    dig_active[0]= 0xFF; // 
    dig_active[1]= 0xFF; // 
    dig_active[6]= 0xFF; // 
    dig_active[7]= 0xFF; // 
    dig_active[4]= 0x85; // d
    dig_active[5]= 0xC5; // o
    dig_active[6]= 0xD5; // n
    dig_active[7]= 0x61; // E
  }
  
  
  for(x=0; x <= 7; x++)
  {
    if(dig_active[x] !=0)
    {
      shiftOut(DIO, CLK, LSBFIRST, dig_active[x]); // Ziffer 0
      shiftOut(DIO, CLK, LSBFIRST, digit[x]);// erstes Digit
      delay(t_display);
    }
  }
}

void taste_pressed(void) // schau ob eine Taste gedrückt ist
{
  timer_key_old=millis();
  T_runs=0;// schalte counter während der Eingabe ab
  if((!digitalRead(SW_m)) || (!digitalRead(SW_s))) // wenn Taste gedrückt
  {
    // schaue ob taste immer noch gedrückt und warte bis wieder losgelassen
    delay(debounce);// um zu "entprellen", wenn noch auf low, dann ist es kein Störsignal
    if(!digitalRead(SW_m)) 
      { taste_m=true; Serial.println("M");}
    else
      { taste_m=false; }
      
    if(!digitalRead(SW_s))
      { taste_s=true; Serial.println("S"); }
    else
      { taste_s=false; }
    do
    { delay(3); }
    while( !((digitalRead(SW_s)) && (digitalRead(SW_m))) );// warte bis beide Tasten wieder losgelassen ACHTUNG KLAMMERN !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    delay(debounce);// wartezeit ist nicht unbedingt notwendig
    key_in=true; // zeige an, dass ein Tastendruck erfolgte und somit ein Count Down aktiv ist
    if(taste_s == true)//wenn Sekunde erhöht werden soll
      { s++; Serial.print("bin: ");Serial.print('s');    Serial.print(s,DEC);}
    if((s > 59) && (m < max_Minuten))//wenn Sekunde erhöht werden soll
    { 
      s=0;
      m++; 
    }
    if((taste_m == true) && (m < max_Minuten))//wenn Sekunde erhöht werden soll
      { m++; }
   }// ende von  if((!taste_s) && (!taste_m)) // wenn Taste gedrückt
}


void fertig(void)        // wenn Zeit abgelaufen dann soll es piepsen
{
  T_runs=0;// schalte Timer ab
  tone(LSP, LSP_Frequenz, LSP_Dauer);
  key_in=0; // nun wird  auf eine erneute Eingabe gewartet
}


void dekrementiere_Zeit(void) // zähle 1 s herunter
{
  
  if(s > 0)
    { s--; }
  else
  {
    if(m > 0)
    {
        m--;
        s=59;
    }
    else // nun ist s==0 und m==0 !
      { fertig(); }
  }
}

void LED_blink(void)
{
  digitalWrite(LED,HIGH);
  delay(300);
  digitalWrite(LED,LOW);
}

 
 
