#include <Arduino.h>
#include <TM1637Display.h>

// Module connection pins (Digital Pins)
#define CLK 2
#define DIO 3

// The amount of time (in milliseconds) between tests
#define TEST_DELAY   2000
#define c_loop 20  // wie oft wird loop durchlaufen bis 1s vergangen ist
#define dauer 1000 // eine Sekunde
#define debounce 50 // wartezeit für nächsten Tastendruck
#define SW_s 4
#define SW_m 5

#define pos 0 // wird ausgegeben ab Position 0
#define len 4 // die Zahl ist 4 Byte lang
#define lead_zero 1 // führende 0 einfügen: ja
#define dots 0b01000000 // doppelpunkt anzeigen
#define no_dots 0b00000000 // keinen Doppelpunkt anzeigen

unsigned long keypressed; // wird mit jedem durchlauf inkrementiert
unsigned long start;
unsigned long timer_akt;
unsigned long timer_old;
int m, s,Zeit;
boolean taste_m, taste_s;
boolean dobledot;
boolean T_runs;


const uint8_t SEG_DONE[] = {
	SEG_B | SEG_C | SEG_D | SEG_E | SEG_G,           // d
	SEG_A | SEG_B | SEG_C | SEG_D | SEG_E | SEG_F,   // O
	SEG_C | SEG_E | SEG_G,                           // n
	SEG_A | SEG_D | SEG_E | SEG_F | SEG_G            // E
	};

const uint8_t SEG_xxxx[] = {
  SEG_G,
  SEG_G,
  SEG_G,
  SEG_G
};

TM1637Display display(CLK, DIO);  //erzeuge ein Objekt display  der Klasse TM1637Display 
  /*
  // Selectively set different digits
  data[0] = display.encodeDigit(0);
  data[1] = display.encodeDigit(1);
  data[2] = display.encodeDigit(2);
  data[3] = display.encodeDigit(3);
  display.setBrightness(7,false);
  display.clear();
  display.setSegments(data+2, 2, 1);
  delay(TEST_DELAY);
  display.showNumberDec(-5, false, 3, 0); // Expect: _-5_
 */
//****************************************************************************************
//**************  S  E  T  U  P  ( )   ***************************************************
//****************************************************************************************  
void setup()
{
  Serial.begin(115200);
  pinMode(SW_s, INPUT_PULLUP);
  pinMode(SW_m, INPUT_PULLUP);
  m = 0;
  s = 0;
  display.setBrightness(0x0f);
  // All segments on
  display.setSegments(SEG_xxxx);
  timer_old=millis();
  dobledot=0; // doppelpunkt ist ausgeschaltet
  T_runs=0;// timer ist nicht aktiv
}
//****************************************************************************************
//**************  L   O   O   P    ( ) ***************************************************
//****************************************************************************************  
void loop()
{
// ist der timer aktiviert? T_runs == 1 ???
// nein: warte auf eingabe: lese Tasten
// ja:
//    ist der timer abgelaufen Zeit == 0 ???
//    ja: done -> Display, piepston, 
//    nein: dec s

// aktuallisiere s und m

// wenn die Zeit > 0 und länger als 1s keine Eingabe erfolgte, soll T_runs aktiviert werden;  c_loop muss dafür den richtigen Wert bekommen

Serial.println(millis(),DEC);

if(T_runs == 0)// wenn timer nicht aktiv, warte auf Eingabe
{
   taste_m = digitalRead(SW_m);
   taste_s = digitalRead(SW_s);

   if((!taste_m) && (taste_s))  // wurde nur taste m gedrückt?
   {
    delay(debounce);
    taste_s = digitalRead(SW_s);
    taste_m = digitalRead(SW_m);
    if((!taste_m) && (taste_s))
    { 
      if(m < 90)
        { m++; }
    }
    while(!digitalRead(SW_m));// warte bis losgelassen
    delay(debounce);
    keypressed=0;
   }

  if((!taste_s) && (taste_m)) // wurde nur Taste s gedrückt ?
  {
    delay(debounce);
    taste_s = digitalRead(SW_s);
    taste_m = digitalRead(SW_m);
    if((!taste_s) && (taste_m))
    {
      if(s < 60)
        {s++;}
      else
      {
        s=0;
        if(m < 90)
          { m++; }
      }
    }
   while(!digitalRead(SW_s));// warte bis losgelassen
   delay(debounce);
   keypressed=0;
  }  
}

keypressed++;
if((keypressed > c_loop)  && (T_runs == 0)) // wenn loop so oft durchlaufen wurde bis 1s vergangen & T_runs==0 (damit nur einmal die Bdinung erfüllt ist
{ T_runs=1; Serial.println(millis(),DEC);}  // uns somit nur einmal ausgeführt wird

if(T_runs == 1) // wenn Timer bereits aktiv ist
{
  if(Zeit > 0)// timer wird dekrementiert und
  {           // Zeit aktualisiert
    Zeit = m * 100 + s;
  }
  else       //  timer ist bereits abgelaufen
  {
    // stelle "done" imd display dar und aktiviere Piepston
    display.setBrightness(7,true);
    display.setSegments(SEG_DONE);
  }
}
  timer_akt=millis();
  if(timer_akt > timer_old+500)
  { 
    dobledot = !dobledot;
    timer_old=millis();
    if(dobledot)
      { 
        if(T_runs==0) {display.setBrightness(3,true);} // nur wenn timer abgelaufen soll displayhelligkeit schwanken
        else
        { display.showNumberDecEx(Zeit,dots,lead_zero,len,pos);s++; }
      }
    else
      {
        display.setBrightness(7,true); 
        if (T_runs ==1) {display.showNumberDecEx(Zeit,no_dots,lead_zero,len,pos);}// nur wenn timer aktiv, soll Zeit angezeigt werden
      }
  }
  
  
  
  /*
// B E I D E   T a s t e n    g e d r ü c k t  !!!
  taste_m = digitalRead(SW_m);
  taste_s = digitalRead(SW_s);
 
  if((!taste_s) && (!taste_m))
  {
    delay(debounce);
    taste_s = digitalRead(SW_s);
    taste_m = digitalRead(SW_m);
    if((!taste_s) && (!taste_m))
    { 
    
    }
    while((!digitalRead(SW_s)) &&(!digitalRead(SW_m)));// warte bis beide Tasten wieder losgelassen
    delay(debounce);// wartezeit ist nicht unbedingt notwendig
  }
  */
}


 
 
